#![feature(try_blocks, proc_macro_hygiene)]
#![deny(rust_2018_idioms, nonstandard_style)]
#![deny(clippy::doc_markdown, clippy::match_bool, clippy::match_same_arms)]
#![warn(future_incompatible)]

use cli::{
  args::{Opts, SubCommand},
  bundle, serve,
};
use collective::cli::ConfigurableAppOpts;
use espresso::bundle::packager;
use snafu::{ResultExt, Snafu};
use std::sync::Arc;

pub mod cli;

#[derive(Snafu, Debug)]
pub enum Error {
  ServeError { source: serve::Error },
  BundleError { source: packager::Error },
  CliError { source: collective::cli::CliError },
}

type Result<T, E = Error> = std::result::Result<T, E>;

#[actix_rt::main]
async fn main() -> Result<()> {
  #[cfg(feature = "console-subscriber")]
  console_subscriber::init();

  let result = inner_main().await;

  if let Err(Error::CliError {
    source: collective::cli::CliError::ArgParse(inner),
  }) = &result
  {
    inner.exit();
  }

  result
}

async fn inner_main() -> Result<()> {
  let (opts, config) = Opts::try_init_with_config().context(CliError)?;

  let config = Arc::new(config);

  match opts.subcmd {
    SubCommand::Serve => serve::serve(config).await.context(ServeError),
    SubCommand::Bundle(opts) => bundle::bundle(config, opts).await.context(BundleError),
  }
}
